﻿using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Data.Entity;

namespace AutofactMVC.Models
{
    public class SchoolContext : DbContext
    {
        public SchoolContext()
            : base("DefaultConnection")
        {
        }

        public DbSet<Student> Students { get; set; }
        public DbSet<StudentScore> StudentScores { get; set; }
        public DbSet<StudentHealthy> StudentHealthies { get; set; }

        protected override void OnModelCreating(DbModelBuilder modelBuilder)
        {
            modelBuilder.Entity<StudentScore>()
                .HasKey(s => s.StudentId);

            modelBuilder.Entity<Student>()
                .HasRequired(s => s.Score)
                .WithRequiredPrincipal(sc => sc.Student);

            modelBuilder.Entity<StudentHealthy>()
               .HasKey(s => s.StudentId);

            modelBuilder.Entity<Student>()
                .HasRequired(s => s.Healthy)
                .WithRequiredPrincipal(sh => sh.Student);
        }
    }

    [Table("Student")]
    public class Student
    {
        [Key]
        public int Id { get; set; }
        public string Name { get; set; }
        public int Age { get; set; }

        public virtual StudentScore Score { get; set; }
        public virtual StudentHealthy Healthy { get; set; }
    }

    public class StudentScore
    {
        public int Score { get; set; }
        public int StudentId { get; set; }
        [ForeignKey("StudentId")]
        public virtual Student Student { get; set; }
    }

    public class StudentHealthy
    {
        public int Score{ get; set; }
        public int StudentId { get; set; }
        [ForeignKey("StudentId")]
        public virtual Student Student { get; set; }
    }
}
