﻿using System;
using System.Collections.Generic;
using System.Linq;
using AutofactMVC.Models;
using AutofactMVC.Repository.Core;

namespace AutofactMVC.Repository
{
    public class StudentExtensionRepository : IStudentExtensionRepository
    {
        private readonly IStudentRepository _studentRepository;
        private readonly IRepository<StudentScore> _studentScoreRepository;

        public StudentExtensionRepository(IRepositoryCenter repositoryCenter)
        {
            _studentRepository = repositoryCenter.GetRepository<IStudentRepository>();
            _studentScoreRepository = repositoryCenter.GetRepository<IRepository<StudentScore>>();
        }
        
        public IEnumerable<StudentExtension> GetStudents()
        {
            var result = new List<StudentExtension>();

            var students = _studentRepository.GetStudents().ToList();
            //取得score信息
            var studentsIds = students.Select(s => s.Id);
            var scores = _studentScoreRepository.Filter(s => studentsIds.Contains(s.StudentId)).ToList();

            foreach (var student in students)
            {
                var temp = new StudentExtension
                               {
                                   Student = student,
                                   StudentScore = scores.FirstOrDefault(s => s.StudentId == student.Id)
                               };
                result.Add(temp);
            }
            return result;
        }
    }

    public class StudentExtensionRepository1 : IStudentExtensionRepository1
    {
        private readonly IStudentRepository _studentRepository;
        private readonly IRepository<StudentScore> _studentScoreRepository;
        private readonly IRepository<StudentHealthy> _studentHealthyRepository;

        public StudentExtensionRepository1(IRepositoryCenter repositoryCenter)
        {
            _studentRepository = repositoryCenter.GetRepository<IStudentRepository>();
            _studentScoreRepository = repositoryCenter.GetRepository<IRepository<StudentScore>>();
            _studentHealthyRepository = repositoryCenter.GetRepository<IRepository<StudentHealthy>>();
        }

        public IEnumerable<StudentExtension1> GetStudents()
        {
            var result = new List<StudentExtension1>();

            var students = _studentRepository.GetStudents().ToList();
            var studentsIds = students.Select(s => s.Id);
            //存储Score查询的结果，避免多次访问数据库来获取Score信息
            List<StudentScore> scoreListTemp = null;
            Func<int, StudentScore> getScoreFunc = id =>
                                                       {
                                                           //第一个Student来获取Score信息的时候，scoreListTemp会是null, 这个时候，会去访问数据库获取Score信息
                                                           //第二个以及以后的Student获取Score信息的时候，scoreListTemp已经有值了, 这样就不会再次访问数据库
                                                           if (scoreListTemp == null)
                                                           {
                                                               scoreListTemp =
                                                                   _studentScoreRepository.Filter(
                                                                       s => studentsIds.Contains(s.StudentId)).ToList();
                                                           }
                                                           return scoreListTemp.FirstOrDefault(s => s.StudentId == id);
                                                       };

            //存储Healthy查询的结果，避免多次访问数据库来获取Healthy信息
            List<StudentHealthy> healthyListTemp = null;
            Func<int, StudentHealthy> getHealthyFunc = id =>
                                                           {
                                                               if (healthyListTemp == null)
                                                               {
                                                                   healthyListTemp =
                                                                       _studentHealthyRepository.Filter(
                                                                           s => studentsIds.Contains(s.StudentId)).
                                                                           ToList();
                                                               }
                                                               return
                                                                   healthyListTemp.FirstOrDefault(s => s.StudentId == id);
                                                           };

            foreach (var student in students)
            {
                var id = student.Id;
                var temp = new StudentExtension1
                {
                    Student = student,
                    StudentScoreLazy = new Lazy<StudentScore>(() => getScoreFunc(id)),
                    StudentHealthyLazy = new Lazy<StudentHealthy>(() => getHealthyFunc(id))
                };
                result.Add(temp);
            }
            return result;
        }
    }


    public class StudentExtension
    {
        public Student Student { get; set; }
        public StudentScore StudentScore { get; set; }
    }

    public class StudentExtension1
    {
        public Student Student { get; set; }
        //Lazy属性
        public Lazy<StudentScore> StudentScoreLazy { get; set; }
        //非Lazy属性，从Lazy属性中取值。当真正用到该属性的时候，会触发数据库访问
        public StudentScore StudentScore { get { return StudentScoreLazy.Value; } }

        public Lazy<StudentHealthy> StudentHealthyLazy { get; set; }
        public StudentHealthy StudentHealthy { get { return StudentHealthyLazy.Value; } }
    }
}